require 'optparse'

require 'messenger_pigeon/version'
require 'messenger_pigeon/pigeon'
require 'messenger_pigeon/config'
require 'messenger_pigeon/endpoint_manager'
require 'messenger_pigeon/cli'

require 'messenger_pigeon/modules/orgmode'
require 'messenger_pigeon/modules/console'
require 'messenger_pigeon/modules/redmine'
require 'messenger_pigeon/modules/csv'

# Messenger Pigeon entry
module MessengerPigeon
  # A flock of Pigeons
  # Manage configuration and dispatch
  class Flock
    def initialize(opts, args)
      @c = Config.new(opts[:config], args)
      @endpoints = {
        source: EndpointManager.new(:source),
        target: EndpointManager.new(:target)
      }
    end

    def release
      @c.conf[:pigeon].each do |name, instructions|
        next unless @c.release_pigeon? name
        release_pigeon prepare_endpoints(instructions), instructions
      end
      @endpoints.values.each(&:finalise)
    end

    private

    def prepare_endpoints(instructions)
      @endpoints.collect do |k, v|
        v.prepare instructions[k], @c.conf[k][instructions[k]]
      end
    end

    def release_pigeon(eps, instructions)
      pigeon = Pigeon.new(*eps, instructions)
      pigeon.fly
    end
  end
end
