module MessengerPigeon
  # MessengerPigeon Configuration
  class Config
    attr_reader :conf

    def initialize(config, pigeons = [])
      @conf = {}
      @pigeons = pigeons
      update config
    end

    def release_pigeon?(name)
      @pigeons.empty? || @pigeons.include?(name)
    end

    private

    def update(new_conf)
      temp_conf = @conf.clone
      case new_conf
      when String
        temp_conf.merge!(load_file new_conf)
      when Hash
        temp_conf.merge! new_conf
      else
        fail "Unsupported configuration datatype: #{new_conf.class}"
      end
      apply_config temp_conf
    end

    # TODO: should do a whole lot more checking. Config pretty expressive and
    # we want to catch problems ASAP.
    def apply_config(config)
      [:pigeon, :target, :source].each do |setting|
        fail "Configuration invalid: #{setting.to_s}" unless
          config[setting] && config[setting].class == Hash &&
          !config[setting].keys.empty?
      end
      @conf = config
    end

    def load_file(config)
      binding.eval(File.read(File.expand_path(config)))
    end
  end
end
