module MessengerPigeon
  # Create and manage instances of Sources or Destinations
  class EndpointManager
    attr_reader :instances

    def initialize(direction)
      @instances = {}
      @direction = direction.to_s.capitalize
    end

    def prepare(name, endpoint_defn)
      fail 'Endpoint definition not defined' unless endpoint_defn.class == Hash
      type = endpoint_defn[:type] || fail('Endpoint type not set')
      @instances[name] || build_instance(name, type, endpoint_defn[:options])
    end

    def direction
      @direction.downcase.to_sym
    end

    def finalise
      @instances.values.each(&:complete)
      @instances = {}
    end

    private

    def build_instance(name, type, options)
      o = Kernel.const_get("MessengerPigeon").const_get("#{type}").const_get("#{@direction}")
      @instances[name] = o.new(options)
    end
  end
end
