require 'csv'
CSVLib = CSV
module MessengerPigeon
  # CSV source/target module
  module CSV
    # Source definition
    class Source
      def initialize(options)
        # :file_glob
        @files = Dir.glob(File.expand_path options[:file_glob])
        @on_complete = options[:on_complete]
      end

      def read
        @files.collect { |f| read_file f }.flatten
      end

      def complete
        case @on_complete
        when :archive
          archive_files
        end
      end

      private

      def archive_files
        @files.each do |f|
          archive_dir = (File.dirname f) + '/archive/'
          Dir.mkdir archive_dir unless Dir.exist? archive_dir
          File.rename(f, archive_dir + (File.basename f)) if File.exist? f
        end
      end

      def read_file(file)
        d = CSVLib.read file
        headings = d.first.collect do |v|
          title = v.gsub(/[^a-z]+/i, '_')
          title.downcase.intern
        end
        d[1..-1].collect { |v| (headings.zip v).to_h }
      end
    end

    # Target definition
    class Target
      def initialize
        fail 'Not Implemented'
      end
    end
  end
end
