

module MessengerPigeon
  # OrgMode source/target module
  module OrgMode
    # Source definition
    class Source
      def initialize(_options = {})
        fail 'Not Implemented'
      end
    end

    # Target definition
    class Target
      def initialize(options = nil)
        @options = options
        open_file File.expand_path(options[:file])
        @known_headings = known_headings
      end

      def filedata
        (@filedata.join "\n") + "\n"
      end

      def known_headings
        res = []
        @filedata.each do |l|
          /^\*+\ (?<heading>.+?)\ +(:[^ ]*:)?$/ =~ l
          res.push heading if heading
        end
        res
      end

      def find_target(headings, star_count, start_line, end_line)
        if headings.empty?
          # Found the target
          return start_line + 1
        end
        target_heading = headings.pop
        @filedata[start_line..end_line].each do |line|
          if /^\*{#{star_count}}\ #{target_heading}(\ +(:[^ ]*:)?)?$/ =~ line
            return find_target(headings, star_count + 1, start_line, find_next_heading(star_count, start_line, end_line))
          end
          start_line += 1
        end
        # Left-over headings
        ([target_heading] + headings).each do |heading|
          @filedata.insert end_line, "#{'*' * star_count} #{heading}"
          star_count += 1
          end_line += 1
        end
        end_line
      end

      def find_next_heading(star_count, start_line, end_line)
        @filedata[start_line + 1..end_line].each do |line|
          if /^\*{#{star_count}}\ (?<heading>.+?)\ +(:[^ ]*:)?$/ =~ line
            return start_line
          end
          start_line += 1
        end
        end_line
      end

      def update(data)
        selector = @options[:heading_selector] % data
        headings = selector.split(@options[:level_separator])
        star_count = 1
        target = find_target headings.reverse, star_count, 0, @filedata.length
        data_string = @options[:data_format].call data
        @filedata.insert target, data_string
      end

      def complete
        @fd.seek 0
        @fd.write filedata
        @fd.close
      end

      private

      def open_file file
        if File.exist? file
          @fd = File.open(file, 'r+')
          @fd.seek 0
          @filedata = @fd.read nil
        else
          @fd = File.open(file, 'w')
          @filedata = ''
        end
        @filedata = @filedata.split(/[\n\r]/)
      end
    end
  end
end
