require 'active_resource'

module MessengerPigeon
  # Redmine source/target module
  module Redmine
    # Module mixin for redmine activeresource init
    module Init
      @@known_instances = {}

      def initialize(options)
        @options = options
        o = get_resource options[:resource]
        o.site = options[:site]
        o.user = options[:user]
        o.password = options[:password]
        o.include_root_in_json = true
        o.format = JsonFormatter.new
        @resource = o
      end

      private

      def get_resource(name)
        return @@known_instances[name] unless @@known_instances[name].nil?
        ar_cl = Class.new ActiveResource::Base
        @@known_instances[name] = Object.const_set(name, ar_cl)
      end
    end

    # Handle JSON returned by Redmine
    class JsonFormatter
      include ActiveResource::Formats::JsonFormat

      def decode(json)
        remove_root(ActiveSupport::JSON.decode(json))
      end

      private

      def remove_root(data)
        # Just return the first value in the hash
        data.each do |_k, v|
          return v
        end
      end
    end

    # Source definition
    class Source
      include Init

      def read
        if @options[:mode] == :specific
          [@resource.find(@options[:key]).attributes]
        elsif @options[:mode] == :all
          read_all
        end
      end

      def complete
      end

      private

      def read_all
        res = @resource.find(:all, params: @options[:params])
        res.map do |r|
          attrs = r.attributes
          attrs.each do |k, v|
            # Convert 1 level of nested attributes
            attrs[k] = v.attributes if v.respond_to? :attributes
          end
        end
      end
    end

    # Target definition
    class Target
      include Init

      def update(data)
        m = @resource.new data
        $stderr.puts issue.errors.full_messages unless m.save
      end

      def complete
      end
    end
  end
end
